%%%%%%%%%%%%%%%%               MATLAB script            %%%%%%%%%%%%%%%%%%
%  for Q 13 : How to convert filters calculated in the frequency domain  %
%             to those in the time domain?                               %
%  in Acoustics ABC pedia published by Coronasha                         %
%     http://www.coronasha.co.jp/np/isbn/9784339008951/   (in Japanese)  %
%                                                                        %
%  wrieten by Takuma OKAMOTO,                                            %
%            National Institute of Communications and Technology, japan  %
%     http://www.okamotocamera.com                                       %
%                                                          9, Feb. 2017. %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;

pg=1;              % print graphs : if pg==1, eps files are generated
save_dir = ['./']; % saving graphs directory

Fs=16000;          % Sampling frequency [Hz]
N=16;              % Number of samples

%%%%%%%%%%%%%%%%%%%%%%% Fig. 1 (Temporal response) %%%%%%%%%%%%%%%%%%%%%%%%
dh=zeros(N,1);
dh(4,1)=1;
figure('Position',[50 650 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(dh,'ko','MarkerSize',15,'MarkerFaceColor','k');
axis([1 N 0 1]);
xlabel('Sample','FontSize',20,'FontName','Times');
ylabel('Amplitude','FontSize',20,'FontName','Times');
set(gca, 'FontSize' ,20,'FontName','Times');
if pg
    print('-depsc',[save_dir 'imp.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Fig. 1 (Real part a) %%%%%%%%%%%%%%%%%%%%%%%%%
df=[0:Fs/(N):Fs]'/1000; % Frequency interval
DH=fft(dh,N);

figure('Position',[700 1100 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(df(2:N/2,1),real(DH(2:N/2,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','k');
hold on;
plot(df(N/2+2:N,1),real(DH(N/2+2:N,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','w');
plot([df(1,1); df(N/2+1,1)],[real(DH(1,1)); real(DH(N/2+1,1))],'ko', ...
                       'MarkerSize',15,'Marker','^','MarkerFaceColor','w');
axis([df(1,1) df(N,1) -1 1]);
xlabel('Frequency [kHz]','FontSize',20,'FontName','Times');
ylabel('Amplitude','FontSize',20,'FontName','Times');
set(gca, 'FontSize' ,20,'FontName','Times');
if pg 
    print('-depsc',[save_dir 'real.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%% Fig. 1 (Imaginary part b) %%%%%%%%%%%%%%%%%%%%%%%
figure('Position',[700 800 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(df(2:N/2,1),imag(DH(2:N/2,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','k');
hold on;
plot(df(N/2+2:N,1),imag(DH(N/2+2:N,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','w');
plot([df(1,1); df(N/2+1,1)],[real(DH(1,1)); imag(DH(N/2+1,1))], ...
                  'ko','MarkerSize',15,'Marker','^','MarkerFaceColor','w');
axis([df(1,1) df(N,1) -1 1]);
xlabel('Frequency [kHz]','FontSize',20,'FontName','Times');
ylabel('Amplitude','FontSize',20,'FontName','Times');
set(gca, 'FontSize' ,20,'FontName','Times');
if pg
    print('-depsc',[save_dir 'imag.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%% Fig. 1 (Amplitude spectrum r) %%%%%%%%%%%%%%%%%%%%%%%
figure('Position',[700 500 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(df(2:N/2,1),abs(DH(2:N/2,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','k');
hold on;
plot(df(N/2+2:N,1),abs(DH(N/2+2:N,1)),'ko','MarkerSize',15, ... 
                                                    'MarkerFaceColor','w');
plot([df(1,1); df(N/2+1,1)],[abs(DH(1,1)); abs(DH(N/2+1,1))], ...
                  'ko','MarkerSize',15,'Marker','^','MarkerFaceColor','w');
axis([df(1,1) df(N,1) 0 1]);
xlabel('Frequency [kHz]','FontSize',18,'FontName','Times');
ylabel('Amplitude','FontSize',18,'FontName','Times');
set(gca, 'FontSize' ,18,'FontName','Times');
if pg
    print('-depsc',[save_dir 'abs.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%% Fig. 1 (Phase spectrum r) %%%%%%%%%%%%%%%%%%%%%%%%
figure('Position',[700 200 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(df(2:N/2,1),angle(DH(2:N/2,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','k');
hold on;
plot(df(N/2+2:N,1),angle(DH(N/2+2:N,1)),'ko','MarkerSize',15, ...
                                                    'MarkerFaceColor','w');
plot([df(1,1); df(N/2+1,1)],[angle(DH(1,1)); angle(DH(N/2+1,1))], ...
                  'ko','MarkerSize',15,'Marker','^','MarkerFaceColor','w');
axis([df(1,1) df(N,1) -pi pi]);
xlabel('Frequency [kHz]','FontSize',18,'FontName','Times');
ylabel('Amplitude','FontSize',18,'FontName','Times');
set(gca, 'FontSize' ,18,'FontName','Times');
if pg
    print('-depsc',[save_dir 'angle.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






Fs=48000;               % Sampling frequency [Hz]
N=64;                   % Number of samples
df=[0:Fs/(N):Fs]'/1000; % Frequency interval

A_dB=csvread('a_weight.csv');  % Load A weight (JIS C 1509)
A_coef=10.^(A_dB(:,2)./20);    % Convert log to linear scale

%%%%%%%%%%%%%%%%% Fig. 2 (Frequency response of A weight) %%%%%%%%%%%%%%%%%
x=[0:(Fs/2)/(N/2):Fs/2]';
H=zeros(N/2+1,1);

nb=max(find(x<20))+1;
ne=min(find(x>20000))-1;
H(nb:ne,1)=interp1(A_dB(:,1),A_coef,x(nb:ne,1));


figure('Position',[1400 1100 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(A_dB(:,1),A_coef,'k','LineWidth',4);
hold on;
plot(x(2:N/2,1),H(2:N/2,1),'ko','MarkerSize',15,'MarkerFaceColor','k');
plot([x(1,1); x(N/2+1)],[H(1,1); H(N/2+1,1)],'ko','MarkerSize',15, ...
                                       'MarkerFaceColor','w','Marker','^');
axis([x(1,1) x(end,1) 0 1.2]);
xlabel('Frequency [kHz]','FontSize',18,'FontName','Times');
ylabel('Amplitude','FontSize',18,'FontName','Times');
set(gca, 'FontSize' ,18,'FontName','Times');
if pg
    print('-depsc',[save_dir 'A.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%% Fig. 2 (Amplitude spectrul = real part) %%%%%%%%%%%%%%%%%
H(N/2+2:N,1)=conj(H(N/2:-1:2,1));

figure('Position',[1400 800 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(df(2:N/2,1),abs(H(2:N/2,1)),'ko','MarkerSize',10, ...
                                                    'MarkerFaceColor','k');
hold on;
plot(df(N/2+2:N,1),abs(H(N/2+2:N,1)),'ko','MarkerSize',10, ...
                                                    'MarkerFaceColor','w');
plot([df(1,1); df(N/2+1,1)],[abs(H(1,1)); abs(H(N/2+1,1))], ...
                  'ko','MarkerSize',15,'Marker','^','MarkerFaceColor','w');
axis([df(1,1) df(N,1) 0 1.2]);
xlabel('Frequency [kHz]','FontSize',18,'FontName','Times');
ylabel('Amplitude','FontSize',18,'FontName','Times');
set(gca, 'FontSize' ,18,'FontName','Times');
if pg
    print('-depsc',[save_dir 'abs_A.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%% Fig. 2 (Result of IFFT) %%%%%%%%%%%%%%%%%%%%%%%%%
h=real(ifft(H,N));

figure('Position',[1400 500 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(h,'k','MarkerSize',10,'MarkerFaceColor','k','LineWidth',5);
axis([1 N min(h) max(h)]);
xlabel('Sample','FontSize',18,'FontName','Times');
ylabel('Amplitude','FontSize',18,'FontName','Times');
set(gca, 'FontSize' ,18,'FontName','Times');
if pg
    print('-depsc',[save_dir 'imp_A.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%% Fig. 2 (FIR filter for A weight) %%%%%%%%%%%%%%%%%%%%%%
h_shift=[h(N/2+1:end,1); h(1:N/2,1)];

figure('Position',[1400 200 600 200]);
set(gcf,'PaperPositionMode','auto');
plot(h_shift,'k','MarkerSize',10,'MarkerFaceColor','k','LineWidth',5);
axis([1 N min(h) max(h)]);
xlabel('Sample','FontSize',18,'FontName','Times');
ylabel('Amplitude','FontSize',18,'FontName','Times');
set(gca, 'FontSize' ,18,'FontName','Times');
if pg
    print('-depsc',[save_dir 'imp_A_shift.eps']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

